# coding: utf-8
#

import uiautomator2 as u2
import pytest

# def setup_function():


def test_get_text(sess: u2.Session):
    assert sess.xpath("App").get_text() == "App"


def test_click(sess: u2.Session):
    sess.xpath("App").click()
    assert sess.xpath("Alarm").wait()
    assert sess.xpath("Alarm").exists


def test_all(sess: u2.Session):
    app = sess.xpath('//*[@text="App"]')
    assert app.wait()
    assert len(app.all()) == 1
    assert app.exists


#! /usr/bin/env python
# -*- coding:utf-8 -*-
# Author: ljw

phones = {"1107": "OPPO 1107",
          "15 Plus": "魅族 15 Plus",
          "15": "魅族 15",
          "1501-A02": "360奇酷 F4",
          "1503-A01": "360奇酷 N4",
          "1505-A01": "360奇酷 N4S",
          "1505-A02": "360 N4S",
          "1515-A01": "360奇酷 Q5",
          "16 X": "魅族 16X",
          "1603-A03": "360奇酷 N4A",
          "1605-A01": "360奇酷 N5",
          "1607-A01": "360奇酷 N5S",
          "16th Plus": "魅族 16th Plus",
          "16th": "魅族 16th",
          "1707-A01": "360 N6",
          "1801-A01": "360 N6 Pro",
          "1803-A01": "360 N7 Lite",
          "1807-A01": "360 N7",
          "1809-A01": "360 N7 Pro",
          "2013022": "红米 手机",
          "2014011": "红米 1S",
          "2014501": "红米 1S ",
          "2014813": "红米 2A",
          "401SO": "索尼 Xperia Z3 ",
          "506SH": "夏普 AQUOS Xx3",
          "831C": "HTC One M8",
          "8676-A01": "酷派 大神 Note 3",
          "8681-M02": "360奇酷 青春版",
          "8692-A00": "360奇酷 旗舰版",
          "A0001": "一加 1",
          "A11": "OPPO A11",
          "A31": "OPPO A31",
          "ALE-UL00": "华为 P8 青春版",
          "ALP-AL00": "华为 Mate 10",
          "ANE-AL00": "华为 nova 3e",
          "ARE-AL00": "华为 荣耀 8X Max",
          "ASUS_T00J": "华硕 ZenFone 5",
          "ASUS_X550": "华硕 飞马2 Plus",
          "ASUS_Z00UD": "Zenfone Selfie",
          "ASUS_Z012DE": "华硕 ZenFone 3",
          "ASUS_Z016DA": "华硕 ZenFone 3",
          "ASUS_Z01QD": "华硕 ROG",
          "ATH-AL00": "华为 荣耀 7i",
          "ATH-UL00": "华为 荣耀 7i",
          "ATU-AL10": "华为 畅享 8e",
          "AUM-AL20": "华为 荣耀畅玩 7A",
          "BAC-TL00": "华为 nova 2 Plus",
          "BKL-AL00": "华为 荣耀 V10",
          "BKL-AL20": "华为 荣耀 V10",
          "BLA-AL00": "华为 Mate 10 Pro",
          "BLN-AL10": "华为 荣耀畅玩 6X",
          "BLN-AL30": "华为 荣耀畅玩 6X",
          "BLN-AL40": "华为 荣耀畅玩 6X",
          "BLN-TL10": "华为 荣耀畅玩 6X",
          "BND-AL10": "华为 荣耀畅玩 7X",
          "BTV-W09": "华为 MediaPad M3",
          "C105": "酷派 cool S1",
          "C106": "酷派 cool1",
          "C107-9": "酷派 cool 1C",
          "C2105": "索尼 C2105",
          "C8817D": "华为 荣耀 畅玩4",
          "CAM-AL00": "华为 荣耀畅玩 5A",
          "CAM-TL00": "华为 荣耀畅玩5A",
          "CHE-TL00": "华为 荣耀 畅玩 4X",
          "Che1-CL20": "华为 荣耀畅玩 4X",
          "Che2-TL00M": "华为 荣耀 畅玩 4X",
          "CHM-CL00": "华为 荣耀 4C 电信版",
          "CHM-UL00": "华为 荣耀 4C",
          "CLT-TL01": "华为 P20 Pro",
          "COL-AL10": "华为 荣耀 10",
          "Coolpad 8297-T01": "酷派 大神 F1",
          "Coolpad 8297W": "酷派 大神 F1",
          "Coolpad 8675": "酷派 大神 F2 移动版",
          "Coolpad 8675-A": "酷派 大神 F2",
          "Coolpad 8702D": "酷派 8720D",
          "Coolpad 8705": "酷派 8705",
          "Coolpad Y75": "酷派 锋尚",
          "Coolpad Y803-9": "酷派 锋尚3",
          "Coolpad Y80D": "酷派 锋尚",
          "COR-AL00": "华为 荣耀Play",
          "CUN-TL00": "华为 荣耀畅玩 5",
          "d-02H": "华为 docomo dtab Compact",
          "DE106": "坚果 R1",
          "DIG-AL00": "华为 畅享 6S",
          "DLI-AL10": "华为 荣耀畅玩 6A",
          "DOOV L9": "朵唯 L9",
          "DOOV S2": "朵唯 S2",
          "DRA-AL00": "华为 畅享 8e ",
          "DUA-AL00": "华为 荣耀畅玩 7",
          "DUK-AL20": "华为 荣耀 V9",
          "E6": "金立 E6",
          "E6533": "索尼 Xperia Z3",
          "E6653": "索尼 Xperia Z",
          "E6883": "索尼 Xperia Z5",
          "E6mini": "金立 E6 mini",
          "E7": "金立 E7",
          "EDI-AL10": "华为 荣耀 Note8",
          "EML-AL00": "华为 P20",
          "EVA-AL00": "华为 P9",
          "EVR-AL00": "华为 Mate 20 X",
          "F-01H": "富士通 ARROWS ",
          "F-01J": "富士通 ARROWS",
          "F-02G": "富士通 F-02G",
          "F-02H": "富士通 ARROWS NX ",
          "F-04G": "富士通 ARROWS NX",
          "F-04H": "ARROWS Tab",
          "F100": "金立 F100",
          "F100S": "金立 F100",
          "F103": "金立 F103",
          "F106": "金立 F106",
          "F301": "金立 F301",
          "F8332": "索尼 Xperia XZ",
          "FIG-AL00": "华为 畅享 7S",
          "FLA-AL10": "华为 畅享 8 Plus",
          "FRD-AL00": "华为 荣耀 8",
          "FRD-AL10": "华为 荣耀 8",
          "FS8010": "夏普 AQUOS S2",
          "G620S-UL00": "华为 荣耀 畅玩4",
          "G8232": "索尼 Xperia XZs",
          "GEM-703L": "华为 荣耀 X2",
          "GIONEE F205L": "金立 F205",
          "GIONEE F6L": "金立 F6",
          "GIONEE GN5007": "金立 大金刚 2",
          "GIONEE M7": "金立 M7",
          "GIONEE S10": "金立 S10",
          "GIONEE S11": "金立 S11",
          "GIONEE S11S": "金立 S11s",
          "GN151": "金立 GN151",
          "GN5001S": "金立 金钢",
          "GN5003": "金立 大金钢",
          "GN5005": "金立 金钢 2",
          "GN8002S": "金立 M6 Plus",
          "GN8003": "金立 M6",
          "GN9000": "金立 S5.5",
          "GN9000L": "金立 S5.5L",
          "GN9005": "金立 S5.1",
          "GN9006": "金立 S7",
          "GN9012": "金立 S6 Pro",
          "GRA-A0": "酷派 酷玩 6C",
          "GT-810": "掠夺者8 A5002",
          "GT-I9060I": "三星 Galaxy Grand Neo Plus",
          "GT-I9152": "三星 GT-I9152",
          "GT-I9158P": "三星 Galaxy Mega",
          "GT-I9208": "三星 GT-I9208",
          "GT-I9300": "三星 Galaxy S3",
          "GT-I9500": "三星 GalaxyS4",
          "GT-I9507V": "三星 Galaxy S4",
          "GT-I9508": "三星 Galaxy S4",
          "GT-N7100": "三星 Galaxy Note 2",
          "GT-N7108": "三星 Galaxy Note 2",
          "H30-T00": "华为 荣耀3C",
          "H30-T10": "华为 荣耀3C",
          "H30-U10": "华为 荣耀3C",
          "H60-L01": "华为 荣耀 6",
          "H60-L03": "华为 荣耀 6",
          "H8296": "索尼 Xperia XZ2",
          "Hisense A2": "海信 A2",
          "Hisense E76": "海信 E76",
          "HLA NOTE1-L": "红辣椒 Note",
          "HM 1S": "红米 1S",
          "HM 2A": "红米 2A",
          "HM NOTE 1LTE": "红米 Note",
          "HM NOTE 1S": "红米 Note",
          "HM NOTE 1TD": "红米 Note",
          "HTC 2Q4D200": "HTC U11+",
          "HTC 2Q4R400": "HTC U11 EYEs",
          "HTC 2Q55300": "HTC U12+",
          "HTC 802w": "HTC One M7",
          "HTC 8088": "HTC One max",
          "HTC 9088": "HTC Butterfly S",
          "HTC A9w": "HTC One A9",
          "HTC D10w": "HTC Desire 10 Pro",
          "HTC D816w": "HTC Desire 816",
          "HTC D820u": "HTC D820u",
          "HTC D830u": "HTC Desire 830",
          "HTC Desire EYE": "HTC Desire Eye",
          "HTC M10u": "HTC 10",
          "HTC M8t": "HTC One E8",
          "HTC One A9": "HTC One A9",
          "HTC One M9PLUS": "HTC M9 台版",
          "HTC One X9 dual sim": "HTC One X9",
          "HTC U Ultra": "HTC U Ultra 港版",
          "HTC U-1w": "HTC U Ultra",
          "HTC_M10h": "HTC 10",
          "HTC_M9u": "HTC One M9",
          "HUAWEI CAZ-AL10": "华为 nova",
          "HUAWEI CAZ-TL20": "华为 nova",
          "HUAWEI G606-T00": "华为 G606-T00",
          "HUAWEI G610-U00": "华为 G610",
          "HUAWEI G7-TL00": "华为 G7",
          "HUAWEI G700-T00": "华为 G700-T00",
          "HUAWEI G700-U00": "华为 G700-U00",
          "HUAWEI G750-T01": "华为 荣耀3X",
          "HUAWEI GRA-CL10": "华为 P8 高配版",
          "HUAWEI GRA-TL00": "华为 P8",
          "HUAWEI GRA-UL00": "华为 P8",
          "HUAWEI M2-801W": "华为 MediaPad",
          "HUAWEI M2-A01W": "华为 MediaPad",
          "HUAWEI MLA-AL00": "华为 麦芒 5",
          "HUAWEI MLA-TL10": "华为 G9 Plus",
          "HUAWEI MLA-UL00": "华为 G9 Plus",
          "HUAWEI MT2-L01": "华为 Mate 2",
          "HUAWEI MT7-TL00": "华为 Mate 7",
          "HUAWEI MT7-TL10": "华为 Mate 7",
          "HUAWEI NXT-AL10": "华为 Mate 8",
          "HUAWEI NXT-DL00": "华为 Mate 8",
          "HUAWEI P6 S-U06": "华为 P6 S",
          "HUAWEI P6-T00": "华为 P6",
          "HUAWEI P7-L00": "华为 P7",
          "HUAWEI P7-L05": "华为 P7",
          "HUAWEI P7-L07": "华为 P7",
          "HUAWEI P8max": "华为 P8max",
          "HUAWEI RIO-AL00": "华为 麦芒 4",
          "HUAWEI RIO-CL00": "华为 麦芒 4",
          "HUAWEI RIO-TL00": "华为 G7 Plus",
          "HUAWEI TAG-AL00": "华为 畅享 5S",
          "HUAWEI TAG-TL00": "华为 畅享 5S",
          "HUAWEI TIT-AL00": "华为 畅享 5",
          "HUAWEI VNS-AL00": "华为 G9",
          "HUAWEI VNS-L21": "华为 P9 Lite",
          "HUAWEI VNS-TL00": "华为 G9",
          "HWI-AL00": "华为 nova 2S",
          "HWT31": "华为 au Qua tab",
          "IM-A870L": "泛泰 VEGA",
          "INE-AL00": "华为 nova 3i",
          "JMM-AL00": "华为 荣耀 V9 Play",
          "JSN-AL00a": "华为 荣耀 8X",
          "K011": "华硕 Memo Pad",
          "KIW-AL10": "华为 荣耀畅玩 5X",
          "KIW-TL00H": "华为 荣耀畅玩 5X",
          "KNT-AL10": "华为 荣耀 V8",
          "KNT-TL10": "华为 荣耀 V8",
          "KOB-W09": "华为 荣耀畅玩",
          "KYT31": "KYOCERA au Qua tab",
          "L36h": "索尼 Z",
          "L39h": "索尼 Xperia Z1",
          "L50w": "索尼 Xperia Z2",
          "LA7-L": "小辣椒 7",
          "LDN-AL00": "华为 畅享 8",
          "Le X507": "乐视 乐1s",
          "Le X620": "乐视 乐2",
          "Le X625": "乐视 乐2 Pro",
          "Le X820": "乐视 乐Max 2",
          "Lenovo A788t": "联想 A788t",
          "Lenovo A808t": "联想 黄金斗士A8",
          "Lenovo A808t-i": "联想 黄金斗士A8",
          "Lenovo A850": "联想 A850",
          "Lenovo K30-T": "联想 乐檬 K3",
          "Lenovo K900": "联想 K900",
          "Lenovo K920": "联想 K920",
          "Lenovo L78011": "联想 Z5",
          "Lenovo P2c72": "联想 VIBE P2",
          "Lenovo P70-t": "联想 P70t",
          "Lenovo PB2-690N": "联想 Phab2 Pro",
          "Lenovo S60-t": "联想 S60t",
          "Lenovo S850t": "联想 S850t",
          "Lenovo S868t": "联想 S868t",
          "Lenovo S90-t": "联想 笋尖S90",
          "Lenovo X2-TO": "联想 VIBE X2",
          "Letv X500": "乐视 乐1s",
          "LEX720": "乐视 乐Pro3",
          "LG-D486": "LG D486",
          "LG-D855": "LG G3",
          "LG-E985T": "LG E985T",
          "LG-H818": "LG G4",
          "LG-H860": "LG G5",
          "LG-H968": "LG V10",
          "LG-M250": "LG K10 2017",
          "LGMS210": "LG Aristo MS210",
          "LGT31": "LGT 31",
          "LGV32": "LG G4",
          "LLD-AL00": "华为 荣耀 9 青春版",
          "LLD-AL20": "华为 荣耀 9i",
          "LND-AL30": "华为 荣耀畅玩 7C",
          "LON-AL00": "华为 Mate 9 Pro",
          "LYA-AL00": "华为 Mate 20 Pro",
          "M040": "魅族 MX2",
          "M1 E": "魅族 魅蓝 E",
          "m1 metal": "魅族 魅蓝 Metal",
          "m1 note": "魅族 魅蓝 Note",
          "m1": "魅族 魅蓝",
          "M15": "魅族 M15",
          "M1813": "魅族 V8 高配版",
          "M1816": "魅族 V8 标准版",
          "M1852": "魅族 X8",
          "M2 E": "魅族 魅蓝 E2",
          "m2 note": "魅族 魅蓝 Note 2",
          "m2": "魅族 魅蓝 2",
          "M3 Max": "魅族 魅蓝 Max",
          "m3 note": "魅族 魅蓝 Note 3",
          "M351": "魅族 MX3",
          "M3s": "魅族 魅蓝 3S",
          "M3X": "魅族 魅蓝 X",
          "M463C": "魅族 魅蓝 Note 电信版",
          "M5 Note": "魅族 魅蓝 Note 5",
          "M5": "魅族 魅蓝 5",
          "M5s": "魅族 魅蓝 5S",
          "M6 Note": "魅族 魅蓝 Note 6",
          "M6": "魅族 魅蓝 6",
          "M623C": "中国移动 A1",
          "M651CY": "中国移动 A3",
          "M760": "中国移动 A4s",
          "Meitu M4": "美图 M4",
          "Meizu 6T": "魅族 魅蓝 6T",
          "MEIZU E3": "魅族 魅蓝 E3",
          "Meizu S6": "魅族 魅蓝 S6",
          "MHA-AL00": "华为 Mate 9",
          "MHA-L29": "华为 Mate 9 国际版",
          "MI 2": "小米 2",
          "MI 2A": "小米 2A",
          "MI 3": "小米 3 移动版",
          "MI 3C": "小米 3 电信版",
          "MI 3W": "小米 3 联通版",
          "MI 4LTE": "小米 4 ",
          "MI 4S": "小米 4S",
          "MI 4W": "小米 4 联通版",
          "MI 5": "小米 5",
          "MI 5C": "小米 5C",
          "MI 5s Plus": "小米 5S Plus",
          "MI 5s": "小米 5S",
          "MI 5X": "小米 5X",
          "MI 6": "小米 6",
          "MI 6X": "小米 6X",
          "MI 8 Explorer Edition": "小米 8 透明探索版",
          "MI 8 Lite": "小米 8 青春版",
          "MI 8 SE": "小米 8 SE",
          "MI 8 UD": "小米 8",
          "MI 8": "小米 8",
          "Mi A1": "小米 A1",
          "MI MAX 2": "小米 MAX 2",
          "MI MAX 3": "小米 Max 3",
          "MI MAX": "小米 Max",
          "Mi Note 2": "小米 Note 2",
          "Mi Note 3": "小米 Note 3",
          "MI NOTE LTE": "小米 Note",
          "MI NOTE Pro": "小米 Note 顶配版",
          "MI PAD 2": "小米 平板2",
          "MI PAD 3": "小米 平板3",
          "MI PAD 4 PLUS": "小米 平板4 Plus",
          "MI PAD 4": "小米 平板4",
          "MI PAD": "小米 平板",
          "Mi-4c": "小米 4C 标准版",
          "MI-ONE Plus": "小米 1",
          "MIX 2": "小米 MIX 2",
          "MIX 2S": "小米 MIX 2S",
          "MIX 3": "小米 MIX 3",
          "MIX": "小米 MIX",
          "Moto E (4)": "Moto E",
          "Moto G (5) Plus": "Moto G5 Plus",
          "Moto G (5)": "Moto G5",
          "Moto X Pro": "Moto X Pro",
          "MP1503": "美图 M6",
          "MX4 Pro": "魅族 MX4 Pro",
          "MX4": "魅族 MX4",
          "MX5": "魅族 MX5",
          "MX6": "魅族 MX6",
          "MYA-AL10": "华为 荣耀畅玩 6",
          "MYA-L22": "华为 Y5",
          "N1": "诺基亚 N1",
          "N1T": "OPPO N1T",
          "N5207": "OPPO N3",
          "NCE-AL00": "华为 畅享 6",
          "NCE-AL10": "华为 畅享 6",
          "NCE-TL10": "华为 畅享 6",
          "NEM-AL10": "华为 荣耀 畅玩5C",
          "NEM-TL00": "华为 荣耀 畅玩5C",
          "NEM-TL00H": "华为 荣耀 畅玩5C",
          "NEO-AL00": "华为 Mate RS",
          "Nexus 4": "Google Nexus 4",
          "Nexus 5": "Google Nexus 5",
          "Nexus 5X": "Google Nexus 5X",
          "Nexus 6": "Google Nexus 6",
          "Nexus 6P": "Google Nexus 6P",
          "Nexus 9": "Google Nexus 9",
          "Nokia 7 plus": "诺基亚 7 Plus",
          "Nokia 8 Sirocco": "诺基亚 8 Sirocco",
          "Nokia X5": "诺基亚 X5",
          "Nokia X6": "诺基亚 X6",
          "NTS-AL00": "华为 荣耀 Magic",
          "NX403A": "努比亚 Z5S mini",
          "NX508J": "努比亚 Z9",
          "NX513J": "努比亚 My",
          "NX529J": "努比亚 Z11 mini",
          "NX531J": "努比亚 Z11",
          "NX563J": "努比亚 Z17",
          "NX569J": "努比亚 Z17 mini ",
          "NX573J": "努比亚 M2",
          "NX575J": "努比亚 N2",
          "NX595J": "努比亚 Z17S",
          "NX609J": "努比亚 红魔手机",
          "NXT-AL10": "华为 Mate 8",
          "OC105": "锤子 坚果 3",
          "OD103": "锤子 坚果 Pro",
          "OE106": "锤子 坚果 Pro 2S",
          "ONE A2001": "一加 2",
          "ONEPLUS A3000": "一加 3",
          "ONEPLUS A3010": "一加 3T",
          "ONEPLUS A5000": "一加 5",
          "ONEPLUS A5010": "一加 5T",
          "ONEPLUS A6000": "一加 6",
          "OPPO A30": "OPPO A30",
          "OPPO A33": "OPPO A33",
          "OPPO A33m": "OPPO A33",
          "OPPO A37m": "OPPO A37",
          "OPPO A53": "OPPO A53",
          "OPPO A53m": "OPPO A53",
          "OPPO A57": "OPPO A57",
          "OPPO A59m": "OPPO A59",
          "OPPO A59s": "OPPO A59s",
          "OPPO A73": "OPPO A73",
          "OPPO A77": "OPPO A77",
          "OPPO A79": "OPPO A79",
          "OPPO A83": "OPPO A83",
          "OPPO R11 Plus": "OPPO R11 Plus",
          "OPPO R11 Plusk": "OPPO R11 Plus",
          "OPPO R11": "OPPO R11",
          "OPPO R11s Plus": "OPPO R11s Plus",
          "OPPO R11s": "OPPO R11s",
          "OPPO R11st": "OPPO R11s",
          "OPPO R11t": "OPPO R11",
          "OPPO R7": "OPPO R7",
          "OPPO R7s": "OPPO R7s ",
          "OPPO R7sm": "OPPO R7s 全网通",
          "OPPO R9 Plustm A": "OPPO R9 Plus",
          "OPPO R9m": "OPPO R9",
          "OPPO R9s Plus": "OPPO R9s Plus",
          "OPPO R9s": "OPPO R9s",
          "OPPO R9sk": "OPPO R9s",
          "OPPO R9tm": "OPPO R9",
          "OS105": "锤子 坚果 Pro 2",
          "PAAM00": "OPPO R15",
          "PADM00": "OPPO A3",
          "PAFM00": "OPPO Find X",
          "PAR-AL00": "华为 nova 3",
          "PBAM00": "OPPO A5",
          "PBEM00": "OPPO R17",
          "PE-CL00": "华为 荣耀 6 Plus",
          "PE-TL10": "华为 荣耀 6 Plus",
          "PH-1": "Essential Phone",
          "PIC-AL00": "华为 nova 2",
          "Pixel 2 XL": "Google Pixel 2 XL",
          "Pixel XL": "Google Pixel XL",
          "Pixel": "Google Pixel",
          "PLAYER": "小辣椒 Player",
          "PLK-CL00": "华为 荣耀 7",
          "PLK-TL01H": "华为 荣耀 7",
          "PLK-UL00": "华为 荣耀 7",
          "POCOPHONE F1": "POCOPHONE F1",
          "PP5600": "PPTV M1",
          "PRA-AL00": "华为 荣耀 8 青春版",
          "PRA-AL00X": "华为 荣耀 8 青春版",
          "PRO 5": "魅族 Pro 5",
          "PRO 6 Plus": "魅族 Pro 6 Plus",
          "PRO 6": "魅族 Pro 6",
          "PRO 6s": "魅族 Pro 6s",
          "PRO 7 Plus": "魅族 Pro 7 Plus",
          "PRO 7-S": "魅族 Pro 7",
          "R2017": "OPPO R2017",
          "R6007": "OPPO R6007",
          "R7007": "OPPO R3",
          "R7c": "OPPO R7 电信版",
          "R7Plus": "OPPO R7 Plus 移动版",
          "R7Plusm": "OPPO R7 Plus 全网通",
          "R8107": "OPPO R8107",
          "R819T": "OPPO R819T",
          "R8207": "OPPO R1C",
          "R821T": "OPPO R821T",
          "R831S": "OPPO R831s",
          "R831T": "OPPO R831T",
          "ramos MOS 1": "蓝魔 MOS1",
          "ramos MOS1max": "蓝魔 MOS1 MAX",
          "Redmi 3": "红米 3",
          "Redmi 3S": "红米 3S",
          "Redmi 3X": "红米 3X",
          "Redmi 4": "红米 4",
          "Redmi 4A": "红米 4A",
          "Redmi 4X": "红米 4X",
          "Redmi 5 Plus": "红米 5 Plus",
          "Redmi 5": "红米 5",
          "Redmi 5A": "红米 5A",
          "Redmi 6 Pro": "红米 6 Pro",
          "Redmi 6A": "红米 6A",
          "Redmi Note 2": "红米 Note 2",
          "Redmi Note 3": "红米 Note 3",
          "Redmi Note 4": "红米 Note 4",
          "Redmi Note 4X": "红米 Note 4X",
          "Redmi Note 5": "红米 Note 5",
          "Redmi Note 5A": "红米 Note 5A",
          "Redmi Pro": "红米 Pro",
          "Redmi S2": "红米 S2",
          "RNE-AL00": "华为 麦芒 6",
          "RVL-AL09": "华为 荣耀 Note10",
          "S39h": "索尼 Xperia C",
          "S9": "金立 S9",
          "SAMSUNG-SM-G930A": "三星 Galaxy S7 美版",
          "SC-03J": "三星 Galaxy S8",
          "SC-04E": "三星 Galaxy S4",
          "SC-04F": "三星 Galaxy S5",
          "SC-04G": "三星 Galaxy S6 Edg",
          "SC-04J": "三星 Galaxy Feel",
          "SC-05G": "三星 Galaxy S6",
          "SCH-I939I": "三星 Galaxy S3 Neo+",
          "SCL-AL00": "华为 荣耀 4A",
          "SCV31": "三星 Galaxy S6 Edge",
          "SCV35": "三星 Galaxy S8+",
          "SCV36": "三星 Galaxy S8",
          "SD4930UR": "Amzon Fire Phone",
          "SGH-N075T": "三星 Galaxy J",
          "SKR-A0": "黑鲨 游戏手机",
          "SLA-AL00": "华为 畅享 7",
          "SM-A3000": "三星 Galaxy A3",
          "SM-A5000": "三星 Galaxy A5",
          "SM-A5100": "三星 Galaxy A5",
          "SM-A520S": "三星 Galaxy A5",
          "SM-A7000": "三星 Galaxy A7",
          "SM-C5010": "三星 Galaxy C5 Pro",
          "SM-C7000": "三星 Galaxy C7",
          "SM-C7010": "三星 Galaxy C7 Pro",
          "SM-C7100": "三星 Galaxy C8",
          "SM-C9000": "三星 Galaxy C9 Pro",
          "SM-E7000": "三星 Galaxy E7",
          "SM-G5309W": "三星 Galaxy GRAND",
          "SM-G530H": "三星 Galaxy Grand Prime ",
          "SM-G532F": "三星 Grand Prime Plus",
          "SM-G5500": "三星 Galaxy On5",
          "SM-G6000": "三星 Galaxy On7",
          "SM-G7108V": "三星 Galaxy Grand 2",
          "SM-G8508S": "三星 Galaxy Alpha",
          "SM-G8750": "三星 Galaxy S ",
          "SM-G9006W": "三星 Galaxy S5",
          "SM-G9008V": "三星 Galaxy S5",
          "SM-G900V": "三星 Galaxy S5",
          "SM-G910S": "三星 Galaxy Round",
          "SM-G9200": "三星 Galaxy S6",
          "SM-G920F": "三星 Galaxy S6",
          "SM-G920V": "三星 Galaxy S6",
          "SM-G9250": "三星 Galaxy S6 Edge",
          "SM-G925V": "三星 Galaxy S6 Edge",
          "SM-G9280": "三星 Galaxy S6 Edge+",
          "SM-G9300": "三星 Galaxy S7",
          "SM-G9308": "三星 Galaxy S7",
          "SM-G930F": "三星 Galaxy S7",
          "SM-G9350": "三星 Galaxy S7 Edge",
          "SM-G935F": "三星 Galaxy S7 Edge",
          "SM-G935V": "三星 Galaxy S7 Edge",
          "SM-G9500": "三星 Galaxy S8",
          "SM-G950F": "三星 Galaxy S8",
          "SM-G9550": "三星 Galaxy S8+",
          "SM-G955N": "三星 Galaxy S8+",
          "SM-G9600": "三星 Galaxy S9",
          "SM-G960N": "三星 Galaxy S9 ",
          "SM-G960U": "三星 Galaxy S9",
          "SM-G965N": "三星 Galaxy S9+",
          "SM-G965U": "三星 Galaxy S9+",
          "SM-J5008": "三星 Galaxy J5",
          "SM-J7008": "三星 Galaxy J7",
          "SM-J701F": "三星 Galaxy J7",
          "SM-J7109": "三星 Galaxy J7",
          "SM-J730GM": "三星 Galaxy J7 Pro",
          "SM-N9002": "三星 Galaxy Note 3",
          "SM-N9008V": "三星 Galaxy Note 3",
          "SM-N900A": "三星 Galaxy Note 3",
          "SM-N9100": "三星 Galaxy Note 4",
          "SM-N910U": "三星 Galaxy Note 4",
          "SM-N9200": "三星 Galaxy Note 5",
          "SM-N9208": "三星 Galaxy Note 5",
          "SM-N920K": "三星 Galaxy Note 5",
          "SM-N9500": "三星 Galaxy Note 8",
          "SM-N950N": "三星 Galaxy Note 8",
          "SM-N950U1": "三星 Galaxy Note 8",
          "SM-N9600": "三星 Galaxy Note 9",
          "SM-P601": "三星 Galaxy Note 10.1",
          "SM-T110": "三星 Galaxy Tab3",
          "SM-T113": "三星 Tab E Lite",
          "SM-T310": "三星 GALAXY Tab3",
          "SM-T560": "三星 Galaxy Tab E",
          "SM-T580": "三星 Galaxy Tab A",
          "SM-T817V": "三星 Galaxy Tab S2",
          "SM701": "锤子 T1",
          "SM901": "锤子 M1",
          "SM919": "锤子 M1L",
          "SNE-AL00": "华为 麦芒 7",
          "SO-03G": "索尼 Xperia Z4",
          "SO-04H": "索尼 Xperia X Performance",
          "SO-04J": "索尼 Xperia XZ Premium",
          "SOV31": "索尼 Xperia Z4",
          "SOV35": "索尼 Xperia XZs",
          "STF-AL00": "华为 荣耀 9",
          "STF-AL10": "华为 荣耀 9",
          "TA-1000": "诺基亚 6",
          "TA-1041": "诺基亚 7",
          "TA-1052": "诺基亚 8",
          "TCL 520": "TCL 520",
          "TCL 750": "TCL 750",
          "TRT-AL00": "华为 畅享 7 Plus",
          "U10": "魅族 魅蓝 U10",
          "V1732A": "vivo Y81s",
          "V1809A": "vivo X23",
          "V1813A": "vivo Y97",
          "V182": "金立 V182",
          "V183": "金立 V183",
          "V185": "金立 V185",
          "V188S": "金立 V188S",
          "V8": "天语 V8",
          "VCR-A0": "酷派 酷玩 6",
          "VIE-AL10": "华为 P9 Plus",
          "Vivo 5R": "BLU vivo 5R",
          "vivo NEX A": "vivo NEX 标准版",
          "vivo NEX S": "vivo NEX 旗舰版",
          "vivo V3": "vivo V3",
          "vivo V3M A": "vivo V3M",
          "vivo V3Max A": "vivo V3Max",
          "vivo V3Max": "vivo V3Max",
          "vivo X20A": "vivo X20",
          "vivo X20Plus A": "vivo X20 Plus",
          "vivo X20Plus UD": "vivo X20 PlusUD",
          "vivo X20Plus": "vivo X20 Plus",
          "vivo X21A": "vivo X21",
          "vivo X21i A": "vivo X21i",
          "vivo X21UD A": "vivo X21UD",
          "vivo X3L": "vivo X3L",
          "vivo X3t": "vivo X3T",
          "vivo X5L": "vivo X5L",
          "vivo X5M": "vivo X5M",
          "vivo X5Max V": "vivo X5 Max V",
          "vivo X5Max+": "vivo X5Max+",
          "vivo X5Pro D": "vivo X5Pro",
          "vivo X5Pro V": "vivo X5Pro V",
          "vivo X5S L": "vivo X5 SL",
          "vivo X6A": "vivo X6A",
          "vivo X6D": "vivo X6D",
          "vivo X6Plus A": "vivo X6 Plus A",
          "vivo X6Plus D": "vivo X6 Plus",
          "vivo X6Plus L": "vivo X6 Plus",
          "vivo X6S A": "vivo X6S",
          "vivo X6SPlus D": "vivo X6S Plus",
          "vivo X7": "vivo X7",
          "vivo X7Plus": "vivo X7 Plus",
          "vivo X9": "vivo X9",
          "vivo X9i": "vivo X9",
          "vivo X9Plus": "vivo X9 Plus",
          "vivo X9s Plus": "vivo X9s Plus",
          "vivo X9s": "vivo X9s",
          "vivo Xplay3S": "vivo Xplay 3S",
          "vivo Xplay5A": "vivo Xplay5",
          "vivo Xplay6": "vivo Xplay6",
          "vivo Y11i T": "vivo Y11iT",
          "vivo Y13L": "vivo Y13L",
          "vivo Y23L": "vivo Y23L",
          "vivo Y27": "vivo Y27",
          "vivo Y29L": "vivo Y29L",
          "vivo Y31A": "vivo Y31A",
          "vivo Y33": "vivo Y33",
          "vivo Y35": "vivo Y35L",
          "vivo Y35A": "vivo Y35A",
          "vivo Y35L": "vivo Y35L",
          "vivo Y37": "vivo Y37",
          "vivo Y51A": "vivo Y51A",
          "vivo Y53": "vivo Y53",
          "vivo Y55": "vivo Y55A",
          "vivo Y66": "vivo Y66",
          "vivo Y67": "vivo Y67",
          "vivo Y67A": "vivo Y67",
          "vivo Y69A": "vivo Y69",
          "vivo Y71A": "vivo Y71",
          "vivo Y75": "vivo Y75",
          "vivo Y75s": "vivo Y75s",
          "vivo Y79A": "vivo Y79",
          "vivo Y83A": "vivo Y83（HIH-PHO-3360）",
          "vivo Y85A": "vivo Y85",
          "vivo Y913": "vivo Y13L",
          "vivo Z1": "vivo Z1（HIH-PHO-3368）",
          "vivo Z1i": "vivo Z1i（HIH-PHO-3506）",
          "vivo": "vivo Y75",
          "VKY-AL00": "华为 P10 Plus",
          "VTR-AL00": "华为 P10",
          "W800": "金立 天鉴 w800",
          "WAS-AL00": "华为 nova 青春版",
          "X9007": "OPPO Find 7 轻装版",
          "X9077": "OPPO Find 7",
          "XT1052": "Moto X（一代）",
          "XT1060": "Moto X（一代）",
          "XT1079": "Moto G 二代",
          "XT1085": "Moto X 二代",
          "XT1096": "Moto X 二代",
          "XT1570": "Moto X Style",
          "XT1635-03": "Moto Z Play",
          "XT1650": "Moto Z",
          "XT1650-05": "Moto Z",
          "XT1662": "Moto M（XT1662）",
          "XT1710-08": "Moto Z2 Play",
          "XT1789-05": "Moto Z 2018",
          "XT1799-2": "Moto 青柚",
          "XT907": "Moto XT907",
          "Y13L": "vivo Y13L",
          "Y23L": "vivo Y23L",
          "Y35A": "vivo Y35A",
          "YQ601": "锤子 坚果手机",
          "Z999": "中兴 Axon M",
          "ZTE A0722": "中兴 Blade A4",
          "ZTE BA610C": "中兴 远航4",
          "ZTE BV0710": "中兴 V7 MAX",
          "ZTE BV0720": "中兴 Blade A2",
          "ZTE BV0730": "中兴 Blade A2 Plus",
          "ZTE C2017": "中兴 天机7Max",
          "ZTE C880U": "中兴 Blade A1",
          "ZTE U930HD": "中兴 U930 HD",
          "ZTE V0900": "中兴 Blade V9",
          "ZTE V975": "中兴 V975",
          "ZUK Z1": "ZUK Z1（Z1221）",
          "ZUK Z2121": "ZUK Z2 Pro（Z2121）",
          "ZUK Z2131": "ZUK Z2",
          "ZUK Z2151": "ZUK Edge"
          }



# coding: utf-8
#

import time

try:
    import Tkinter as tk
except ImportError:
    import tkinter as tk


def retryskipabort(message, timeout=20):
    """
    Show dialog of RETRY,SKIP,ABORT
    Returns:
        one of "retry", "skip", "abort"
    """
    root = tk.Tk()
    root.geometry("400x200")
    root.title("Exception handle")
    root.eval('tk::PlaceWindow %s center' % root.winfo_pathname(root.winfo_id()))
    root.attributes("-topmost", True)

    _kvs = {"result": "abort"}

    def cancel_timer(*args):
        root.after_cancel(_kvs['root'])
        root.title("Manual")

    def update_prompt():
        cancel_timer()

    def f(result):
        def _inner():
            _kvs['result'] = result
            root.destroy()
        return _inner

    tk.Label(root, text=message).pack(side=tk.TOP, fill=tk.X, pady=10)

    frmbtns = tk.Frame(root)
    tk.Button(frmbtns, text="Skip", command=f('skip')).pack(side=tk.LEFT)
    tk.Button(frmbtns, text="Retry", command=f('retry')).pack(side=tk.LEFT)
    tk.Button(frmbtns, text="ABORT", command=f('abort')).pack(side=tk.LEFT)
    frmbtns.pack(side=tk.BOTTOM)

    prompt = tk.StringVar()
    label1 = tk.Label(root, textvariable=prompt) #, width=len(prompt))
    label1.pack()

    deadline = time.time() + timeout

    def _refresh_timer():
        leftseconds = deadline - time.time()
        if leftseconds <= 0:
            root.destroy()
            return
        root.title("Test will stop after " + str(int(leftseconds)) + " s")
        _kvs['root'] = root.after(500, _refresh_timer)

    _kvs['root'] = root.after(0, _refresh_timer)
    root.bind('<Button-1>', cancel_timer)
    root.mainloop()

    return _kvs['result']


if __name__ == '__main__':
    print(retryskipabort('LKJSDF\nlkjj\what?lkjsdlfjaskdfjlasdkjflnice'))



# coding: utf-8
#

import six
import functools

from uiautomator2.exceptions import (
    SessionBrokenError,
    UiObjectNotFoundError)


def U(x):
    if six.PY3:
        return x
    return x.decode('utf-8') if type(x) is str else x


def E(x):
    if six.PY3:
        return x
    return x.encode('utf-8') if type(x) is unicode else x


def check_alive(fn):
    @functools.wraps(fn)
    def inner(self, *args, **kwargs):
        if not self.running():
            raise SessionBrokenError(self._pkg_name)
        return fn(self, *args, **kwargs)

    return inner


def hooks_wrap(fn):
    @functools.wraps(fn)
    def inner(self, *args, **kwargs):
        name = fn.__name__.lstrip('_')
        self.server.hooks_apply("before", name, args, kwargs, None)
        ret = fn(self, *args, **kwargs)
        self.server.hooks_apply("after", name, args, kwargs, ret)
    return inner


# Will be removed in the future
def wrap_wait_exists(fn):
    @functools.wraps(fn)
    def inner(self, *args, **kwargs):
        timeout = kwargs.pop('timeout', self.wait_timeout)
        if not self.wait(timeout=timeout):
            raise UiObjectNotFoundError({
                'code': -32002,
                'message': E(self.selector.__str__())
            })
        return fn(self, *args, **kwargs)

    return inner


def intersect(rect1, rect2):
    top = rect1["top"] if rect1["top"] > rect2["top"] else rect2["top"]
    bottom = rect1["bottom"] if rect1["bottom"] < rect2["bottom"] else rect2[
        "bottom"]
    left = rect1["left"] if rect1["left"] > rect2["left"] else rect2["left"]
    right = rect1["right"] if rect1["right"] < rect2["right"] else rect2[
        "right"]
    return left, top, right, bottom


class Exists(object):
    """Exists object with magic methods."""

    def __init__(self, uiobject):
        self.uiobject = uiobject

    def __nonzero__(self):
        """Magic method for bool(self) python2 """
        return self.uiobject.jsonrpc.exist(self.uiobject.selector)

    def __bool__(self):
        """ Magic method for bool(self) python3 """
        return self.__nonzero__()

    def __call__(self, timeout=0):
        """Magic method for self(args).

        Args:
            timeout (float): exists in seconds
        """
        if timeout:
            return self.uiobject.wait(timeout=timeout)
        return bool(self)

    def __repr__(self):
        return str(bool(self))



# coding: utf-8
#
# https://github.com/openatx/u2init#rest-api

import re
import time
import requests
import uiautomator2
from logzero import logger
import logging

logger.setLevel(logging.DEBUG)

# def oppo_install(self, apk_url):
#     m = hashlib.md5()
#     m.update(apk_url.encode('utf-8'))
#     key = m.hexdigest()[8:16]
#     dst = "/sdcard/atx-" + key + ".apk"  # 安装包会在安装完后自动删除
#     d = uiautomator2.connect(self._device_url)
#     print(d.info, dst)
#     d.push_url(apk_url, dst)
#     # d.push("/Users/codeskyblue/workdir/atxcrawler/apks/ApiDemos-debug.apk", dst) # For debug
#     with d.session("com.coloros.filemanager") as s:
#         s(text=u"所有文件").click()
#         s(className="android.widget.ListView").scroll.to(textContains=key)
#         s(textContains=key).click()

#         btn_done = d(className="android.widget.Button", text=u"完成")
#         while not btn_done.exists:
#             s(text="继续安装旧版本").click_exists()
#             s(text="无视风险安装").click_exists()
#             s(text="重新安装").click_exists()
#             # 自动清除安装包和残留
#             if s(resourceId=
#                  "com.android.packageinstaller:id/install_confirm_panel"
#                  ).exists:
#                 # 通过偏移点击<安装>
#                 s(resourceId=
#                   "com.android.packageinstaller:id/bottom_button_layout"
#                   ).click(offset=(0.75, 0.5))
#         btn_done.click()

#     def vivo_install(self, apk_url):
#         print("Vivo detected, open u2 watchers")
#         u = uiautomator2.connect_wifi(self._device_url)
#         u.watcher("AUTO_INSTALL").when(
#             textMatches="好|安装", className="android.widget.Button").click()
#         u.watchers.watched = True
#         self.pm_install(apk_url)


def install_apk(device_url, apk_url):
    """
    Args:
        device_url: udid, device_ip or serial(when usb connected)
    """
    psurl = pkgserv_addr(device_url)
    _http_install_apk(psurl, apk_url)


def pkgserv_addr(device_url):
    """
    根据设备线获取到atxserver的地址，然后再获取到u2init的地址，直接再决定是无线安装还是手动安装
    
    Returns:
        Package API url
    """
    logger.info("device url %s", device_url)
    d = uiautomator2.connect(device_url)
    devinfo = d.device_info
    serial = devinfo['serial']
    logger.info("serial %s, udid %s", serial, devinfo['udid'])
    aserver_url = devinfo.get(
        "serverUrl",
        "http://wifiphone.nie.netease.com")  # TODO(atx-agent should udpate)
    logger.info("atx-server url %s", aserver_url)
    r = requests.get(
        aserver_url + "/devices/" + devinfo['udid'] + "/info").json()
    pvd = r.get('provider')
    if not pvd:
        logger.info("u2init not connected")
        return "http://" + d.wlan_ip + ":7912/packages"
    pkg_url = 'http://%s:%d/devices/%s/pkgs' % (pvd['ip'], pvd['port'], serial)
    logger.info("package url %s", pkg_url)
    return pkg_url


def _http_install_apk(pkg_restapi, apk_url):
    """ install apk from u2init """
    resp = requests.post(pkg_restapi, data={"url": apk_url}).json()
    if not resp.get('success'):
        raise RuntimeError(resp.get('description'))

    id = resp['data']['id']
    logger.info("install id %s", id)
    _wait_installed(pkg_restapi + "/" + id)


def _wait_installed(query_url):
    """ query until install finished """
    while True:
        data = safe_getjson(query_url)
        status = data.get('status')
        logger.debug("%s %s", status, data.get('description'))
        if status in ("success", "failure"):
            break
        time.sleep(1)


def safe_getjson(url):
    """ get rest api """
    r = requests.get(url).json()
    desc = r.get('description')
    if not r.get('success'):
        raise RuntimeError(desc)
    return r.get('data')


def main():
    # ins = U2Installer("http://localhost:17000")
    # apk_url = "http://arch.s3.netease.com/hzdev-appci/h35_trunk_RelWithDebInfo_373397.apk"  # 1.8G large apk
    apk_url = "https://gohttp.nie.netease.com/tools/apks/qrcodescan-2.6.0-green.apk"

    # command line
    # python -m uiautomator2.cli install 10.240.174.43 http://arch.s3.netease.com/hzdev-appci/h35_trunk_RelWithDebInfo_373397.apk -s http://wifiphone.nie.netease.com
    # psurl = pkgserv_addr("3578298f")
    psurl = pkgserv_addr("10.242.163.69")
    install_apk(psurl, apk_url)


if __name__ == '__main__':
    main()



# coding: utf-8
#
# Refs: https://github.com/openatx/uiautomator2/blob/d08e2e8468/uiautomator2/adbutils.py

from __future__ import print_function

import datetime
import os
import re
import socket
import stat
import struct
import subprocess
import time
from collections import namedtuple
from contextlib import contextmanager

import six
import whichcraft

_OKAY = "OKAY"
_FAIL = "FAIL"
_DENT = "DENT"  # Directory Entity
_DONE = "DONE"

DeviceItem = namedtuple("Device", ["serial", "status"])
ForwardItem = namedtuple("ForwardItem", ["serial", "local", "remote"])
FileInfo = namedtuple("FileInfo", ['mode', 'size', 'mtime', 'name'])


def get_free_port():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('localhost', 0))
    try:
        return s.getsockname()[1]
    finally:
        s.close()


def adb_path():
    path = whichcraft.which("adb")
    if path is None:
        raise EnvironmentError(
            "Can't find the adb, please install adb on your PC")
    return path


class AdbError(Exception):
    """ adb error """


class _AdbStreamConnection(object):
    def __init__(self, host=None, port=None):
        # assert isinstance(host, six.string_types)
        # assert isinstance(port, int)
        self.__host = host
        self.__port = port
        self.__conn = None

        self._connect()

    def _connect(self):
        adb_host = self.__host or os.environ.get("ANDROID_ADB_SERVER_HOST",
                                                 "127.0.0.1")
        adb_port = self.__port or int(
            os.environ.get("ANDROID_ADB_SERVER_PORT", 5037))
        self.__conn
        s = self.__conn = socket.socket()
        s.connect((adb_host, adb_port))
        return self

    def close(self):
        self.conn.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc, traceback):
        self.close()

    @property
    def conn(self):
        return self.__conn

    def send(self, cmd):
        assert isinstance(cmd, six.string_types)
        self.conn.send("{:04x}{}".format(len(cmd), cmd).encode("utf-8"))

    def read(self, n):
        assert isinstance(n, int)
        return self.conn.recv(n).decode()

    def read_raw(self, n):
        assert isinstance(n, int)
        t = n
        buffer = b''
        while t > 0:
            chunk = self.conn.recv(t)
            if not chunk:
                break
            buffer += chunk
            t = n - len(buffer)
        return buffer

    def read_string(self):
        size = int(self.read(4), 16)
        return self.read(size)

    def read_until_close(self):
        content = ""
        while True:
            chunk = self.read(4096)
            if not chunk:
                break
            content += chunk
        return content

    def check_okay(self):
        data = self.read(4)
        if data == _FAIL:
            raise AdbError(self.read_string())
        elif data == _OKAY:
            return
        raise AdbError("Unknown data: %s" % data)


class AdbClient(object):
    def connect(self):
        return _AdbStreamConnection()

    def server_version(self):
        """ 40 will match 1.0.40
        Returns:
            int
        """
        with self.connect() as c:
            c.send("host:version")
            c.check_okay()
            return int(c.read_string(), 16)

    def shell(self, serial, command):
        """Run shell in android and return output
        Args:
            serial (str)
            command: list, tuple or str
        
        Returns:
            str
        """
        assert isinstance(serial, six.string_types)
        if isinstance(command, (list, tuple)):
            command = subprocess.list2cmdline(command)
        assert isinstance(command, six.string_types)
        with self.connect() as c:
            c.send("host:transport:" + serial)
            c.check_okay()
            c.send("shell:" + command)
            c.check_okay()
            return c.read_until_close()

    def forward_list(self, serial=None):
        list_cmd = "host:list-forward"
        if serial:
            list_cmd = "host-serial:{}:list-forward".format(serial)
        with self.connect() as c:
            c.send(list_cmd)
            c.check_okay()
            content = c.read_string()
            for line in content.splitlines():
                parts = line.split()
                if len(parts) != 3:
                    continue
                yield ForwardItem(*parts)

    def forward(self, serial, local, remote, norebind=False):
        """
        Args:
            serial (str): device serial
            local, remote (str): tcp:<port> or localabstract:<name>
            norebind (bool): fail if already forwarded when set to true
        
        Raises:
            AdbError
        
        Protocol: 0036host-serial:6EB0217704000486:forward:tcp:5533;tcp:9182
        """
        with self.connect() as c:
            cmds = ["host-serial", serial, "forward"]
            if norebind:
                cmds.append("norebind")
            cmds.append(local + ";" + remote)
            c.send(":".join(cmds))
            c.check_okay()

    def iter_device(self):
        """
        Returns:
            list of DeviceItem
        """
        with self.connect() as c:
            c.send("host:devices")
            c.check_okay()
            output = c.read_string()
            for line in output.splitlines():
                parts = line.strip().split("\t")
                if len(parts) != 2:
                    continue
                if parts[1] == 'device':
                    yield AdbDevice(self, parts[0])

    def devices(self):
        return list(self.iter_device())

    def must_one_device(self):
        ds = self.devices()
        if len(ds) == 0:
            raise RuntimeError("Can't find any android device/emulator")
        if len(ds) > 1:
            raise RuntimeError(
                "more than one device/emulator, please specify the serial number"
            )
        return ds[0]

    def device_with_serial(self, serial=None):
        if not serial:
            return self.must_one_device()
        return AdbDevice(self, serial)

    def sync(self, serial):
        return Sync(self, serial)


class AdbDevice(object):
    def __init__(self, client, serial):
        self._client = client
        self._serial = serial

    @property
    def serial(self):
        return self._serial

    def __repr__(self):
        return "AdbDevice(serial={})".format(self.serial)

    @property
    def sync(self):
        return Sync(self._client, self.serial)

    def adb_output(self, *args, **kwargs):
        """Run adb command and get its content

        Returns:
            string of output

        Raises:
            EnvironmentError
        """

        cmds = [adb_path(), '-s', self._serial
                ] if self._serial else [adb_path()]
        cmds.extend(args)
        cmdline = subprocess.list2cmdline(map(str, cmds))
        try:
            return subprocess.check_output(
                cmdline, stderr=subprocess.STDOUT, shell=True).decode('utf-8')
        except subprocess.CalledProcessError as e:
            if kwargs.get('raise_error', True):
                raise EnvironmentError(
                    "subprocess", cmdline,
                    e.output.decode('utf-8', errors='ignore'))

    def shell_output(self, *args):
        return self._client.shell(self._serial, subprocess.list2cmdline(args))

    def forward_port(self, remote_port):
        assert isinstance(remote_port, int)
        for f in self._client.forward_list(self._serial):
            if f.serial == self._serial and f.remote == 'tcp:' + str(
                    remote_port) and f.local.startswith("tcp:"):
                return int(f.local[len("tcp:"):])
        local_port = get_free_port()
        self._client.forward(self._serial, "tcp:" + str(local_port),
                             "tcp:" + str(remote_port))
        return local_port

    def push(self, local, remote):
        assert isinstance(local, six.string_types)
        assert isinstance(remote, six.string_types)
        self.adb_output("push", local, remote)

    def install(self, apk_path):
        """
        sdk = self.getprop('ro.build.version.sdk')
        sdk > 23 support -g
        """
        assert isinstance(apk_path, six.string_types)
        self.adb_output("install", "-r", apk_path)

    def uninstall(self, pkg_name):
        assert isinstance(pkg_name, six.string_types)
        self.adb_output("uninstall", pkg_name)

    def getprop(self, prop):
        assert isinstance(prop, six.string_types)
        return self.shell_output('getprop', prop).strip()

    def package_info(self, pkg_name):
        assert isinstance(pkg_name, six.string_types)
        output = self.shell_output('dumpsys', 'package', pkg_name)
        m = re.compile(r'versionName=(?P<name>[\d.]+)').search(output)
        version_name = m.group('name') if m else None
        m = re.search(r'PackageSignatures\{(.*?)\}', output)
        signature = m.group(1) if m else None
        if version_name is None and signature is None:
            return None
        return dict(version_name=version_name, signature=signature)


class Sync():
    def __init__(self, adbclient, serial):
        self._adbclient = adbclient
        self._serial = serial
        # self._path = path

    @contextmanager
    def _prepare_sync(self, path, cmd):
        c = self._adbclient.connect()
        try:
            c.send(":".join(["host", "transport", self._serial]))
            c.check_okay()
            c.send("sync:")
            c.check_okay()
            # {COMMAND}{LittleEndianPathLength}{Path}
            c.conn.send(
                cmd.encode("utf-8") + struct.pack("<I", len(path)) +
                path.encode("utf-8"))
            yield c
        finally:
            c.close()

    def stat(self, path):
        assert isinstance(path, six.string_types)
        with self._prepare_sync(path, "STAT") as c:
            assert "STAT" == c.read(4)
            mode, size, mtime = struct.unpack("<III", c.conn.recv(12))
            return FileInfo(mode, size, datetime.datetime.fromtimestamp(mtime),
                            path)

    def iter_directory(self, path):
        assert isinstance(path, six.string_types)
        with self._prepare_sync(path, "LIST") as c:
            while 1:
                response = c.read(4)
                if response == _DONE:
                    break
                mode, size, mtime, namelen = struct.unpack(
                    "<IIII", c.conn.recv(16))
                name = c.read(namelen)
                yield FileInfo(mode, size,
                               datetime.datetime.fromtimestamp(mtime), name)

    def list(self, path):
        return list(self.iter_directory(path))

    def push(self, src, dst, mode=0o755):
        # IFREG: File Regular
        # IFDIR: File Directory
        assert isinstance(dst, six.string_types)
        path = dst + "," + str(stat.S_IFREG | mode)
        with self._prepare_sync(path, "SEND") as c:
            r = src if hasattr(src, "read") else open(src, "rb")
            try:
                while True:
                    chunk = r.read(4096)
                    if not chunk:
                        mtime = int(time.time())
                        c.conn.send(b"DONE" + struct.pack("<I", mtime))
                        break
                    c.conn.send(b"DATA" + struct.pack("<I", len(chunk)))
                    c.conn.send(chunk)
                assert c.read(4) == _OKAY
            finally:
                if hasattr(r, "close"):
                    r.close()

    def pull(self, path):
        assert isinstance(path, six.string_types)
        with self._prepare_sync(path, "RECV") as c:
            while True:
                cmd = c.read(4)
                if cmd == "DONE":
                    break
                assert cmd == "DATA"
                chunk_size = struct.unpack("<I", c.read_raw(4))[0]
                chunk = c.read_raw(chunk_size)
                if len(chunk) != chunk_size:
                    raise RuntimeError("read chunk missing")
                print("Chunk:", chunk)


adb = AdbClient()

if __name__ == "__main__":
    print("server version:", adb.server_version())
    print("devices:", adb.devices())
    d = adb.devices()[0]

    print(d.serial)
    for f in adb.sync(d.serial).iter_directory("/data/local/tmp"):
        print(f)
    # adb.listdir(d.serial, "/data/local/tmp/")
    finfo = adb.sync(d.serial).stat("/data/local/tmp")
    print(finfo)
    import io
    sync = adb.sync(d.serial)
    sync.push(
        io.BytesIO(b"hi5a4de5f4qa6we541fq6w1ef5a61f65ew1rf6we"),
        "/data/local/tmp/hi.txt", 0o644)
    # sync.mkdir("/data/local/tmp/foo")
    sync.pull("/data/local/tmp/hi.txt")
